# -*- coding: utf-8 -*-
import os
import sys
import time
import codecs
from pprint import pprint

from pyxml2obj import XMLin, XMLout
from resources import RESMAN

def convert_to_dict(filename):
    s = ""
    try:
        f = open(filename)
        for line in f:
            s = s + line
        f.close()
    except:
        print "Could not open file: ", filename
        return None
    return XMLin(s, {"forcearray": 1})

def load_profile(filename):
    profile = convert_to_dict(filename)
    profile = verify(profile)
    return profile


def verify(profile):
    #Unfinished: check if profile has valid format
    cmdlist = profile["command-list"][0]["command"]
    for cmd in cmdlist:
        #cname
        if "cname" not in cmd.keys():
            cmd["cname"] = ""

        #phrase
        if "phrase" not in cmd.keys() or type(cmd["phrase"]) is not list:
            cmd["phrase"] = [""]

        for i in xrange(len(cmd["phrase"])):
            p = cmd["phrase"][i]
            if type(p) not in [str, unicode]:
                cmd["phrase"][i] = u""

        #keys
        if "keys" not in cmd.keys():
            cmd["keys"] = [{ "content":"", "keycodes":"","repeat":"0"}]
        elif type(cmd["keys"]) is dict:
            keycopy = cmd["keys"].copy()
            cmd["keys"] = [keycopy]


        #key details
        for k in cmd["keys"]:
            if "content" not in k.keys():
                k["content"] = u""
            if "repeat" not in k.keys():
                k["repeat"] = u"0"
            if "keycodes" not in k.keys():
                k["keycodes"] = u""

    return profile


def save_as_xml(filename, data_dict):
    header = '<?xml version="1.0" encoding="utf-8" ?>'
    options =  { "xmldecl":header,
                "rootname": "config" }
    try:
        converted = XMLout(data_dict, options)
    except Exception, Error:
        msg = "An error occured while converting the data: \n"
        msg = msg + str(Error)
        print Error
        return msg

    try:
        f = codecs.open(filename, "w", encoding="utf-8")
        f.write(converted)
        f.close()
        return False
    except Exception, Error:
        msg = "An error occured while saving the file: \n"
        msg = msg + str(Error)
        print Error
        return msg

def gen_commands_words(xmldict):
    commands = {}

    #list of command dicts
    cmdlist = []
    for lst in xmldict["command-list"]:
        cmdlist.extend(lst["command"])
    cmds = cmdlist

    #keys: list of keystrokes
    #phrase: voice command
    words = []
    for c in cmds:
        if type(c["phrase"]) is not list:
            c["phrase"] = [c["phrase"]]
        for phrase in c["phrase"]:
            if type(phrase) is str or type(phrase) is unicode:
                commands[phrase.lower().strip()] = c["keys"]
                words.append(phrase.lower())
            else:
                print "Invalid phrase:", phrase
    return commands, words

def print_used_keys(commands):
    keylist = []
    for phrase in commands.keys():
        keystrokes = commands[phrase]
        if len(keystrokes) == 1:
            keystrokes = [keystrokes]
        for key in keystrokes:
            press = key["type"]["value"]
            if press not in keylist:
                keylist.append(press)
    keylist.sort()
    pprint(keylist)

def save_html(filename, string):
    try:
        f = codecs.open(filename, "w", encoding="utf-8")
        f.write(string)
        f.close()
    except Exception, Error:
        msg = "An error occured while saving the file: \n"
        msg = msg + str(Error)
        print Error
        return msg

def export_html(config):
    def gen_tag(tag, content):
        tmp = u"".join(["<", tag, ">", content, "</", tag, ">\n"])
        return tmp

    def join_html(html, string):
        return u"".join([html, string])

    print
    print "Exporting HTML cheat sheet..."

    html = u""

    #headline
    optlist = config["option-list"][0]["option"]
    for o in optlist:
        if "profilename" in o.keys():
            html = join_html(html,
                gen_tag("h1", o["profilename"]))

    cmdlists = config["command-list"]
    cmd_info = ["cname", "phrase", "keys"]
    for lst in cmdlists:
        cat = lst["category"]
        html = join_html(html, "<table class='") #add table
        html = join_html(html, cat)
        html = join_html(html, "'>")

        #category caption
        html = join_html(html, gen_tag("caption", cat))

        #table headers
        head = "<tr><th>Command</th><th>Phrase</th><th>Keys</th></tr>"
        html = join_html(html, head)
        color_even = True

        #command info
        for c in lst["command"]:
            if color_even: tr = "<tr class='odd'>"
            else: tr = "<tr class='even'>"
            html = join_html(html, tr)

            html = join_html(html, "<td class='cname'>") #cmd name
            html = join_html(html, c["cname"][0])
            html = join_html(html, "</td>")

            html = join_html(html, "<td class='phrase'>") #phrases
            for value in c["phrase"]:
                html = join_html(html, "".join([value, "<br />"]))
            html = join_html(html, "</td>")

            html = join_html(html, "<td class='keys'>") #keys
            for value in c["keys"]:
                keypress = value["content"]
                if "repeat" in value.keys() and value["repeat"]!="0":
                    keypress = "".join([keypress, " (", value["repeat"], "x)"])
                html = join_html(html, "".join([keypress, "<br />"]))
            html = join_html(html, "</td></tr>")
            color_even = not color_even

        html = join_html(html, "</table>")  #close table



    html += "</body></html>"

    #add template head
    template = RESMAN["html_templates"][0]
    path = os.path.join(RESMAN["html_template_path"], template)
    head = RESMAN.load_file(path)

    html = "".join([head, html])
    return html


