# -*- coding: utf-8 -*-

import os
import tkFileDialog
import tkMessageBox
from pprint import pprint
from Tkinter import *
from ttk import *
import webbrowser

import effbotwidgets
from effbotwidgets import AutoScrollbar as Scrollbar
from commands import *
from resources import RESMAN
try:
    from littlespeaker import LittleSpeaker
    SPEAKER = LittleSpeaker()
except ImportError:
    print "Could not import LittleSpeaker. Voice testing won't be available."
    SPEAKER = None



#totally useful global variables
VALUES = {}
CONFIG = {}
GLOBALS = {
    "changed": False,
    "icon_scale": 1.0,
}



class App(Frame):
    width = 670
    height = 500

    def __init__(self, filename, master=None):
        Frame.__init__(self)
        self.next_app = None
        self.option_add('*tearOff', FALSE)

        #always stretch content to fit window
        self.grid(sticky=(N,S,E,W))
        self.grid_columnconfigure(0, weight=1)
        self.grid_rowconfigure(0, weight=1)
        top = self.winfo_toplevel()
        top.grid_columnconfigure(0, weight=1)
        top.grid_rowconfigure(0, weight=1)
        top.title("CaptainSays Editor")
        self.top = top

        #load command file
        self.open(filename=filename)

        #create widgets
        self.create_widgets()

        #keyboard shortcuts
        top.bind("<Control-o>", self.open)
        top.bind("<Control-s>", self.save)
        top.bind("<Control-S>", self.saveas)
        top.bind("<F8>", self.export_html)

    def create_widgets(self):
        self.update_menu()


    def create_notebook(self):
        #reinit values
        global VALUES
        VALUES = {}

        row, col = 0,0
        #notebook
        self.notebook = Notebook(self, width=self.width, height=self.height)
        self.notebook.grid(row=row, column=col, sticky=(N,E,S,W))

        #add tabs to notebook
        f0 = ProfileTab(self.notebook, self.profile)
        self.notebook.add(f0, text=f0.title)
        self.tablist = []
        for lst in self.commandlist:
            fx = GenTab(self.notebook, lst)
            self.tablist.append(fx)
            self.notebook.add(fx, text=fx.title)

    def callback(self, ev=None):
        print "Hi. I am a placeholder callback that does nothing."
        print "Ah.. Life is goooood.... "

    def open_new(self, template):
        filename = os.path.join(RESMAN["template_path"], template)
        self.open(filename=filename)
        self.filename = "" #no filename

    def open(self, ev=None, filename=None):
        if not filename:
            filename = tkFileDialog.askopenfilename(
                initialdir=RESMAN["profile_path"],
                title="Open profile",
                filetypes=["XML *.xml"])
        if not filename: return

        try:
            self.config = load_profile(filename)
            self.commandlist = self.config["command-list"]
            self.profile = self.config["option-list"][0]["option"]
        except Exception, error:
            msg = "Could not open file."
            tkMessageBox.showerror(title="Error", message=msg)
            #self.open()
            return
        self.filename = filename
        self.create_notebook()
        self.update_title()

    def update_title(self, ev=None):
        profilename = VALUES["option"]["profilename"].get()
        title = "".join(["CaptainSays Editor - ", profilename])
        self.top.title(title)

    def update_menu(self, ev=None):
        RESMAN.reload_stuff()
        row, col = 0, 0
        #Add menu
        win = self.winfo_toplevel()
        menubar = Menu(win)
        win['menu'] = menubar

        #File-menu
        menu_file = Menu(menubar)

        #new -> select template
        menu_new = Menu(menu_file)
        templates = RESMAN["templates"]
        for t in templates:
            menu_new.add_command(label=t,
                command=lambda template=t: self.open_new(template))
        menu_file.add_cascade(menu=menu_new, label="New")

        #open, save, saveas, quit
        menu_file.add_command(label="Open", command=self.open,
            accelerator="Ctrl+O")
        menu_file.add_separator()
        menu_file.add_command(label="Save", command=self.save,
            accelerator="Ctrl+S")
        menu_file.add_command(label="Save As", command=self.saveas,
            accelerator="Ctrl+Shift+S")
        menu_file.add_separator()
        menu_file.add_command(label="Quit", command=self.top.destroy)
        menubar.add_cascade(menu=menu_file, label="File")

        #Tools-menu
        menu_tools = Menu(menubar)
        menu_tools.add_command(label="Create HTML cheat sheet",
            command=self.export_html, accelerator="F8")
        menubar.add_cascade(menu=menu_tools, label="Tools")

    def saveas(self, ev=None):
        filename = tkFileDialog.asksaveasfilename(
            initialdir=RESMAN["profile_path"],
            title="Save as..",
            filetypes=["XML *.xml"])
        if filename:
            if not "." in filename:
                filename = "".join([filename, ".xml"])
            self.filename=filename
            self.save()
            self.update_menu()

    def save(self, ev=None):
        if not self.filename:
            self.saveas()
            return

        #get values from options widgets
        for opt in self.profile:
            key = opt.keys()[0]
            opt[key] = VALUES["option"][key].get()

        #convert and save as xml file
        error = save_as_xml(self.filename, self.config)
        if not error:
            print "Saved to:", self.filename
        else:
            tkMessageBox.showerror(title="Error", message=error)

    def export_html(self, ev=None):
        ex = export_html(self.config)
        f = "nofilename.html"
        if self.filename:
            f = self.filename.replace(".xml", ".html")
        save_html(f, ex)

        msg = "HTML export finished.\nOpen in browser?"
        answer = tkMessageBox.askquestion(
            title="Open browser?",
            message = msg)
        if answer=="yes": webbrowser.open(f)

class GenTab(Panedwindow):
    title = "General Commands"
    keys = ["cname", "phrase", "keys"]

    def __init__(self, master, commandlist):
        PanedWindow.__init__(self, master, orient=HORIZONTAL)
        self.sel = None
        self.title = commandlist["category"]
        self.commandlist = commandlist
        commands = commandlist["command"]
        self.commands = commands
        self.stringvars = {}

        #first pane: list of command names
        f1 = Labelframe(self, text="Name")
        f1.grid_rowconfigure(1, weight=1)
        f1.grid_columnconfigure(0, weight=1)
        self.add(f1)

        #add/remove command-button
        row, col = 0, 0
        b = Button(f1, text="+", command=self.add_cmd)
        b.grid(column=col, row=row+3, sticky=(N,W))
        b = Button(f1, text="-", command=self.rem_cmd)
        b.grid(column=col+1, row=row+3, sticky=(N,E))

        row += 1
        #create listbox
        VALUES["commandnames"] = StringVar()
        l = Listbox(f1, height=5, listvariable=VALUES["commandnames"],
            selectmode=SINGLE, exportselection=0)
        l.grid(column=col, row=row, columnspan=2, sticky=(N,W,E,S))
        l.bind("<<ListboxSelect>>", self.select_cmd)

        #fill listbox
        for cmd in self.commands:
            name = cmd["cname"]
            if type(name) is list: name = name[0]
            l.insert(END,name)

        #make listbox scrolleable
        s = Scrollbar(f1, orient=VERTICAL, command=l.yview)
        s.grid(column=col+2, row=row, sticky=(N,S,W))
        l['yscrollcommand'] = s.set
        s2 = Scrollbar(f1, orient=HORIZONTAL, command=l.xview)
        s2.grid(column=col, row=row+1, columnspan=2, sticky=(N,S,W,E))
        l['xscrollcommand'] = s2.set

        #second pane: editable command details
        f2 = Labelframe(self, text="Attributes", width=100, height=100)
        self.add(f2)
        self.detailframe = f2
        self.detailframe.grid_rowconfigure(0,weight=1)
        self.detailframe.grid_columnconfigure(0,weight=1)

        #create scrolled frame
        canvas = effbotwidgets.ScrolledCanvas(self.detailframe)
        canvas.grid(row=0, column=0, sticky=(N,S,E,W))
        self.details = effbotwidgets.ScrolledFrame(canvas)
        self.details.grid(row=0, column=0, sticky=(N,S,E,W), padx=10, pady=10)
        self.details.grid_columnconfigure(1, weight=1)
        self.details.grid_rowconfigure(1, weight=1)

        self.detailwidgets = []
        #"add"-buttons
        row, col = 0, 0
        b = Button(self.details, text="Add phrase", width=20)
        b.attr = "phrase"
        b.bind("<Button-1>", self.add_attr)
        b.grid(row=row, column=col, sticky=(N,W))
        b = Button(self.details, text="Add key", width=20)
        b.attr = "keys"
        b.bind("<Button-1>", self.add_attr)
        b.grid(row=row, column=col+1, sticky=(N,W), columnspan=3)

        #select first command
        self.listbox = l
        l.selection_set(0)
        self.sel = 0
        self.select_cmd()

    def get_selection(self, ev=None):
        #get selected item from list
        sel = self.listbox.curselection()
        if not sel: return
        self.sel = int(sel[0])
        self.listbox.selection_set(self.sel)

    def select_cmd(self, ev=None):
        #select and show details about command
        self.get_selection()
        cmd = self.commands[self.sel]

        #destroy widgets
        for w in self.detailwidgets:
            w.destroy()

        #reset StringVars
        self.stringvars = {}
        for key in cmd.keys():
            item = cmd[key]
            self.stringvars[key] = []
            if type(item[0]) is dict:
                #print "dict", item
                for i in item:
                    valuedict = {}
                    for k in i.keys():
                        stringvar = StringVar()
                        stringvar.set(i[k])
                        valuedict[k] = stringvar
                    self.stringvars[key].append(valuedict)
            else:
                self.stringvars[key] = []
                #print "list", item
                for i in item:
                    stringvar = StringVar()
                    stringvar.set(i)
                    self.stringvars[key].append(stringvar)

        #pprint(cmd)
        #pprint(self.stringvars)

        #create widgets
        self.detailwidgets = []
        row, col = 1, 0

        #cname
        l = Label(self.details, text="cname:")
        l.grid(row=row, column=col, padx=1, pady=1, sticky=(N,E,W))
        self.detailwidgets.append(l)
        col += 1
        e = Entry(self.details, textvariable=self.stringvars["cname"][0])
        e.bind("<Any-KeyRelease>", self.update_cmd)
        e.grid(row=row, column=col, padx=1, pady=1, sticky=(N,E,W),
                columnspan=3)
        self.detailwidgets.append(e)
        row, col = row + 1, 0

        #phrase
        l = Label(self.details, text="phrase:")
        l.grid(row=row, column=col, padx=1, pady=1, sticky=(N,E,W))
        self.detailwidgets.append(l)
        col += 1
        count = 0
        for p in self.stringvars["phrase"]:
            col = 1
            e = Entry(self.details, textvariable=p)
            e.bind("<Any-KeyRelease>", self.update_cmd)
            e.grid(row=row, column=col, padx=1, pady=1, sticky=(N,E,W),
                columnspan=3)
            self.detailwidgets.append(e)
            if count > 0:
                b = Button(self.details, text="X",width=1)
                b.cmd_key = ("phrase", count)
                b.bind("<Button-1>", self.remove_attr)
                b.grid(row=row, column=4, sticky=(N,E))
                self.detailwidgets.append(b)
            count += 1
            row += 1
        col = 0

        #keys - labels
        l = Label(self.details, text="keys:")
        l.grid(row=row, column=col, padx=1, pady=1, sticky=(N,W))
        self.detailwidgets.append(l)
        col += 1

        l = Label(self.details, text="keynames")
        l.grid(row=row, column=col, padx=1, pady=1, sticky=(N,W))
        self.detailwidgets.append(l)
        col += 1

        l = Label(self.details, text="codes")
        l.grid(row=row, column=col, padx=1, pady=1, sticky=(N,W))
        self.detailwidgets.append(l)
        col += 1

        l = Label(self.details, text="repeat")
        l.grid(row=row, column=col, padx=1, pady=1, sticky=(N,W))
        self.detailwidgets.append(l)
        col += 1

        row += 1
        #print "-------"
        #print self.stringvars["keys"]
        #print "-------"

        count = 0
        for keypress in self.stringvars["keys"]:
            col = 1
            #keynames
            e = Entry(self.details, textvariable=keypress["content"], width=20)
            e.bind("<Any-KeyRelease>", self.update_cmd)
            e.grid(row=row, column=col, padx=1, pady=1, sticky=(N,E,W))
            self.detailwidgets.append(e)
            col += 1

            #keycodes
            e = Entry(self.details, textvariable=keypress["keycodes"],
                width=10, state=DISABLED)
            e.bind("<Any-KeyRelease>", self.update_cmd)
            e.grid(row=row, column=col, padx=1, pady=1, sticky=(N,E,W))
            self.detailwidgets.append(e)
            col += 1

            #repeat
            if "repeat" not in keypress.keys():
                keypress["repeat"] = StringVar()
            s = Spinbox(self.details, from_=0, to=50, width=3,
                command=self.update_cmd,
                textvariable=keypress["repeat"])
            s.grid(row=row, column=col, sticky=(N,W,E))
            s.bind("<Any-KeyRelease>", self.update_cmd)
            self.detailwidgets.append(s)
            col += 1

            #add key grabbing button
            b = Button(self.details, text="Grab", width=4)
            b.key_nr = count
            b.bind("<Button-1>", self.grab_key)
            b.grid(row=row, column=col, sticky=(N,E,W))
            self.detailwidgets.append(b)
            col += 1

            if count > 0:
                b = Button(self.details, text="X",width=1)
                b.cmd_key = ("keys", count)
                b.bind("<Button-1>", self.remove_attr)
                b.grid(row=row, column=col, sticky=(N,E,W))
                self.detailwidgets.append(b)
            row += 1
            count += 1
        self.details.widgets_aftermath()

    def update_cmd(self, ev=None):
        cmd = self.commands[self.sel]
        #update current command in self.commands variable
        for k in self.stringvars.keys():
            value = self.stringvars[k]

            if type(value[0]) is dict:
                valuelist = []
                for item in value:
                    valuedict = {}
                    for key in item.keys():
                        valuedict[key] = item[key].get()
                    valuelist.append(valuedict)
                cmd[k] = valuelist
            elif type(value) is list:
                cmd[k] = []
                for i in value:
                    cmd[k].append(i.get())
            else:
                cmd[k] = [value.get()]
        self.commandlist["command"][self.sel] = cmd
        self.listbox.delete(self.sel)
        self.listbox.insert(self.sel, cmd["cname"][0])
        self.listbox.selection_set(self.sel)

    def remove_attr(self, ev):
        attr, i = ev.widget.cmd_key

        #get selected item from list
        cmd = self.commands[self.sel]

        #set value in self.commands
        if attr in cmd.keys():
            if i < len(cmd[attr]):
                cmd[attr].pop(i)

        #update details panel
        self.select_cmd()

    def add_attr(self, ev):
        attr = ev.widget.attr

        #get selected item from list
        cmd = self.commands[self.sel]

        #add value in self.commands
        if attr in cmd.keys():
            value = ""
            if attr=="keys":
                value = {
                    "content": "",
                    "keycodes": "",
                    "repeat": "0"
                }
            else:
                value = ""
            cmd[attr].append(value)
        #update details panel
        self.select_cmd()

    def grab_key(self, ev):
        key_nr = ev.widget.key_nr
        cmd = self.commands[self.sel]
        if not type(cmd["keys"]) is list:
            cmd["keys"] = [cmd["keys"]]
        if key_nr >= len(cmd["keys"]): return

        #launch keygrabber
        top = self.winfo_toplevel()
        app = KeyGrabber(self, "Keygrabber")

        #set keys
        cmd["keys"][key_nr]["content"] = app.keys
        cmd["keys"][key_nr]["keycodes"]= app.keycodes

        #update details panel
        self.select_cmd()

    def add_cmd(self, ev=None):
        cmd_template = {
            "cname":["[enter name]"],
            "phrase":[""],
            "keys":[{ "content":"","keycodes":"","repeat":"0"}]
        }

        #add to commandlist
        self.commandlist["command"].append(cmd_template.copy())

        #add to listbox and select it
        self.listbox.insert(END, cmd_template["cname"][0])
        self.listbox.selection_clear(0, END)
        self.listbox.selection_set(END)
        self.get_selection()
        self.select_cmd()

        #scroll to new command on bottom of list
        self.listbox.yview_scroll(len(self.commands), UNITS)

    def rem_cmd(self, ev=None):
        if self.sel is None or self.sel >= len(self.commands)-1: return
        self.commandlist["command"].pop(self.sel)
        self.listbox.delete(self.sel)
        if self.sel > 0: self.sel -= 1
        self.listbox.selection_set(self.sel)
        self.select_cmd()

class ProfileTab(Frame):
    title = "Profile"

    def __init__(self, master, profile):
        Frame.__init__(self, master, padding=5)
        self.grid_columnconfigure(1, weight=1)
        self.profile = profile
        VALUES["option"] = {}
        voice_test = ["greeting"]

        #profile options
        row, col = 0, 0
        for opt in self.profile:
            for k in opt.keys():
                name = k.capitalize() + ":"
                value = opt[k]
                VALUES["option"][k] = StringVar()
                VALUES["option"][k].set(value)

                label = Label(self, text=name, anchor=E)
                label.grid(row=row, column=col, sticky=(W,N), pady=1)
                col += 1

                entry = Entry(self, textvariable=VALUES["option"][k])
                entry.grid(row=row, column=col, sticky=(E,W,N), pady=1)
                if k=="profilename":
                    entry.bind("<Any-KeyRelease>", master.master.update_title)
                col += 1

                if k in voice_test and SPEAKER:
                    #add voice test button
                    funct = lambda key=k: self.voice_test(key)
                    b = Button(self, text="Play", command=funct)
                    b.grid(row=row, column=col)
                    col += 1
                col, row = 0, row + 1

    def voice_test(self, key):
        if not SPEAKER: return
        SPEAKER.say(VALUES["option"][key].get())

class KeyGrabber(effbotwidgets.Dialog):
    #keygrabbing dialog
    def body(self, master):
        def dummy(): print "dumdidum"
        self.keys = None
        self.keycodes = None
        self.press = []
        self.press_count = 0
        Label(master, text="Press keys..").grid()

        #track any key presses and releases
        top = self.winfo_toplevel()
        top.bind("<KeyPress>", self.collect_info)
        top.bind("<KeyRelease>", self.set_info)

    def collect_info(self, ev):
        if not self.press or self.press[-1].keycode != ev.keycode:
            self.press.append(ev)
            self.press_count += 1

        #keypress events shall not be passed on any further!
        #..otherwise trying to bind ALT+F4 would close the dialog
        return "break"

    def set_info(self, ev):
        self.press_count = 0
        if self.press_count<=0:
            txt = ""
            keycodes = ""
            for k in self.press:
                if txt:
                    txt = txt + " + "
                    keycodes = keycodes + " + "
                txt = txt + k.keysym
                keycodes = keycodes + str(k.keycode)
            self.keys = txt
            self.keycodes = keycodes
            self.ok()
        return "break"

    def buttonbox(self): pass
