from Tkinter import *
import os

# Thanks to
# http://effbot.org/tkinterbook/tkinter-dialog-windows.htm
# http://effbot.org/zone/tkinter-autoscrollbar.htm

class AutoScrollbar(Scrollbar):
    # a scrollbar that hides itself if it's not needed.  only
    # works if you use the grid geometry manager.
    def set(self, lo, hi):
        if float(lo) <= 0.0 and float(hi) >= 1.0:
            # grid_remove is currently missing from Tkinter!
            self.tk.call("grid", "remove", self)
        else:
            self.grid()
        Scrollbar.set(self, lo, hi)
    def pack(self, **kw):
        raise TclError, "cannot use pack with this widget"
    def place(self, **kw):
        raise TclError, "cannot use place with this widget"


class ScrolledCanvas(Canvas):
    def __init__(self, master=None):
        vscrollbar = AutoScrollbar(master)
        vscrollbar.grid(row=0, column=1, sticky=N+S+E)
        hscrollbar = AutoScrollbar(master, orient=HORIZONTAL)
        hscrollbar.grid(row=1, column=0, sticky=E+W+S)

        Canvas.__init__(self, master,
                        highlightthickness=0,
                        yscrollcommand=vscrollbar.set,
                        xscrollcommand=hscrollbar.set)

        vscrollbar.config(command=self.yview)
        hscrollbar.config(command=self.xview)

class ScrolledFrame(Frame):
    def __init__(self, scrolledcanvas):
        Frame.__init__(self, scrolledcanvas)
        self.rowconfigure(0, weight=1)
        self.columnconfigure(0, weight=1)
        self.grid(row=0, column=0, sticky=(N,S,E,W))

    def widgets_aftermath(self):
        #call when done adding widgets
        self.master.create_window(0, 0, anchor=NW, window=self)
        self.update_idletasks()
        self.master.config(scrollregion=self.master.bbox("all"))



class Dialog(Toplevel):

    def __init__(self, parent, title = None):

        Toplevel.__init__(self, parent)
        self.transient(parent)

        if title:
            self.title(title)

        self.parent = parent

        self.result = None

        body = Frame(self)
        self.initial_focus = self.body(body)
        body.pack(padx=5, pady=5)

        self.buttonbox()

        self.grab_set()

        if not self.initial_focus:
            self.initial_focus = self

        self.protocol("WM_DELETE_WINDOW", self.cancel)

        self.geometry("+%d+%d" % (parent.winfo_rootx()+50,
                                  parent.winfo_rooty()+50))

        self.initial_focus.focus_set()

        self.wait_window(self)

    #
    # construction hooks

    def body(self, master):
        # create dialog body.  return widget that should have
        # initial focus.  this method should be overridden

        pass

    def buttonbox(self):
        # add standard button box. override if you don't want the
        # standard buttons

        box = Frame(self)

        w = Button(box, text="OK", width=10, command=self.ok, default=ACTIVE)
        w.pack(side=LEFT, padx=5, pady=5)
        w = Button(box, text="Cancel", width=10, command=self.cancel)
        w.pack(side=LEFT, padx=5, pady=5)

        self.bind("<Return>", self.ok)
        self.bind("<Escape>", self.cancel)

        box.pack()

    #
    # standard button semantics

    def ok(self, event=None):

        if not self.validate():
            self.initial_focus.focus_set() # put focus back
            return

        self.withdraw()
        self.update_idletasks()

        self.apply()

        self.cancel()

    def cancel(self, event=None):

        # put focus back to the parent window
        self.parent.focus_set()
        self.destroy()

    #
    # command hooks

    def validate(self):

        return 1 # override

    def apply(self):

        pass # override



if __name__=="__main__":
    root = Tk()
    # create scrolled canvas
    canvas = ScrolledCanvas(root)
    canvas.grid(row=0, column=0, sticky=(N,S,E,W))

    # make the canvas expandable
    root.grid_rowconfigure(0, weight=1)
    root.grid_columnconfigure(0, weight=1)

    # create canvas contents
    frame = ScrolledFrame(canvas)
    rows = 15
    for i in range(1,rows):
        for j in range(1,10):
            button = Button(frame, padx=7, pady=7, text="[%d,%d]" % (i,j))
            button.grid(row=i, column=j, sticky='news')
    frame.widgets_aftermath()
    root.mainloop()
