# -*- coding: utf-8 -*-

import os
import webbrowser
import tkMessageBox
from Tkinter import *
from ttk import *

import effbotwidgets
import configcaptainsays
from effbotwidgets import AutoScrollbar as Scrollbar
from resources import RESMAN
from commands import *


class App(Frame):
    def __init__(self, master=None, speechrec=True):
        Frame.__init__(self, master)
        self.next_app = None
        self.speechrec = speechrec

        #always stretch content to fit window
        self.grid(sticky=(N,S,E,W), padx=15, pady=15)
        self.grid_columnconfigure(0, weight=1)
        self.grid_rowconfigure(0, weight=1)
        top = self.winfo_toplevel()
        top.grid_columnconfigure(0, weight=1)
        top.grid_rowconfigure(0, weight=1)
        top.title("CaptainSays Launcher")
        top.resizable(width=False, height=False)
        self.top = top

        self.create_widgets()

    def create_widgets(self):
        row, col = 0, 0
        #logo
        f = os.path.join(RESMAN["data_path"], "logo.gif")
        self.logo = PhotoImage(file=f)
        l = Label(self, image=self.logo, width=250)
        l.bind("<Button-1>", self.show_about)
        l.grid(row=row, column=col, pady=10, sticky=N)
        row += 1

        #instructional label
        label = Label(self, text="Select profile:")
        label.grid(row=row, column=col, sticky=N)
        row += 1

        #profiles listbox
        l=Listbox(self, height=10, width=40, selectmode=SINGLE, exportselection=0)
        l.grid(column=col, row=row, sticky=(N,W,E,S))
        self.listbox = l

        #make listbox scrolleable
        s = Scrollbar(self, orient=VERTICAL, command=l.yview)
        s.grid(column=col+1, row=row, sticky=(N,S,W))
        l['yscrollcommand'] = s.set
        row += 1
        s2 = Scrollbar(self, orient=HORIZONTAL, command=l.xview)
        s2.grid(column=col, row=row, sticky=(N,S,W,E))
        l['xscrollcommand'] = s2.set
        row += 1

        #fill listbox
        for profile in RESMAN["profiles"]:
            l.insert(END,profile)

        #select first listbox item
        if RESMAN["profiles"]: l.selection_set(0)

        #buttons
        b = Button(self, text="Launch", command=self.launch)
        b.grid(row=row, column=col, columnspan=2, sticky=(E,W))
        if not self.speechrec: b.config(state=DISABLED)
        row += 1

        b = Button(self, text="Edit", command=self.edit)
        b.grid(row=row, column=col, columnspan=2, sticky=(E,W))
        row += 1
        b = Button(self, text="HTML-View", command=self.view)
        b.grid(row=row, column=col, columnspan=2, sticky=(E,W))
        row += 1
        b = Button(self, text="Quit", command=self.quit)
        b.grid(row=row, column=col, columnspan=2, sticky=(E,W))
        row += 1

    def launch(self, ev=None):
        p = self.get_selection()
        self.next_app = ("launch", p)
        self.quit()

    def edit(self, ev=None):
        p = self.get_selection()
        self.next_app = ("edit", p)
        self.quit()

    def view(self, ev=None):
        p = self.get_selection()
        #generate html view
        dct = convert_to_dict(p)
        ex = export_html(dct)
        filename = p.replace(".xml", ".html")
        save_html(filename, ex)
        webbrowser.open(filename)

    def get_selection(self, ev=None):
        #get filepath of selected profile
        sel = self.listbox.curselection()
        if not sel: return None
        sel = int(sel[0])
        p = os.path.join(RESMAN["profile_path"],RESMAN["profiles"][sel])
        self.listbox.selection_set(sel)
        return p

    def quit(self, ev=None):
        self.top.destroy()

    def show_about(self, ev=None):
        AboutApp(self)

    def callback(self, ev=None):
        print "Hi. I'm a placeholder callback. I'm bored."



class AboutApp(effbotwidgets.Dialog):
    def body(self, master):
        #set window title
        top = self.winfo_toplevel()
        top.title("About")

        row, col = 0, 0
        #About image
        f = os.path.join(RESMAN["data_path"], "about.gif")
        self.about_img = PhotoImage(file=f)
        l = Label(master, image=self.about_img)
        l.grid(row=row, column=col)
        row += 1

        #About text
        txt = "Made by zahmeKatzen.\nReleased under GPLv3."
        l = Label(master, text=txt)
        l.grid(row=row, column=col, padx=10, pady=5)
        row += 1

        #link
        url = "http://zahmekatzen.net"
        l = Label(master,
            text=url,
            cursor="hand2",
            foreground="#1010ff")
        l.bind("<Button-1>", lambda x: webbrowser.open(url))
        l.grid(row=row, column=col, padx=10, pady=5)
        row += 1

        #close button
        b = Button(master, text="Close", command=self.ok)
        b.grid(row=row, column=col)

    def buttonbox(self): pass


