# Sample code for speech recognition using the MS Speech API borrowed from Inigo Surguy (inigosurguy@hotmail.com)
#
# Modified for tv watching/media center by Rob Cherry simonsays@lxrb.com
# Hosted at http://code.google.com/p/simonsays/
#
# Further modified by zahmekatzen@gmx.de
# Hosted at some URL XD


# Next 2 lines should avoid having users run makepy on their own
from win32com.client import gencache
gencache.EnsureModule('{C866CA3A-32F7-11D2-9602-00C04F8EE628}', 0, 5, 4)
from win32com.client import constants
import win32com.client
import win32con,win32api,win32gui
import pythoncom
import string
import time

#from Tkinter import *

from profile import PROFILE

COMMANDS = {}
RUN = 1


class SpeechRecognition:
    """ Initialize the speech recognition with the passed in list of words """
    def __init__(self, wordsToAdd):
        # For text-to-speech
        self.speaker = win32com.client.Dispatch("SAPI.SpVoice")
        self.listener = win32com.client.Dispatch("SAPI.SpInProcRecognizer")
        self.listener.AudioInputStream = win32com.client.Dispatch("SAPI.SpMMAudioIn")
        self.context = self.listener.CreateRecoContext()
        self.grammar = self.context.CreateGrammar()
        # Do not allow free word recognition - only command and control
        # recognizing the words in the grammar only
        self.grammar.DictationSetState(0)
        # Create a new rule for the grammar, that is top level (so it begins
        # a recognition) and dynamic (ie we can change it at runtime)
        self.wordsRule = self.grammar.Rules.Add("wordsRule",
                        constants.SRATopLevel + constants.SRADynamic, 0)
        # Clear the rule (not necessary first time, but if we're changing it
        # dynamically then it's useful)
        self.wordsRule.Clear()
        # And go through the list of words, adding each to the rule
        [ self.wordsRule.InitialState.AddWordTransition(None, word) for word in wordsToAdd ]
        # Set the wordsRule to be active
        self.grammar.Rules.Commit()
        self.grammar.CmdSetRuleState("wordsRule", 1)

        # Commit the changes to the grammar
        self.grammar.Rules.Commit()
        # And add an event handler that's called back when recognition occurs
        self.eventHandler = ContextEvents(self.context)

        # Announce we've started - use greeting in Profile
        phrase =  PROFILE.get_option("greeting")
        if phrase is None:
            phrase = "Started successfully"
        self.say(phrase)

    """Speak a word or phrase"""
    def say(self, phrase):
        self.speaker.Speak(phrase)


"""The callback class that handles the events raised by the speech object.
    See "Automation | SpSharedRecoContext (Events)" in the MS Speech SDK
    online help for documentation of the other events supported. """
class ContextEvents(win32com.client.getevents("SAPI.SpSharedRecoContext")):
    phrase = ""
    keypress_duration = 0
    # Note that being here is no guarantee.  Any loud noise will force the speech recognition
    # into thinking that one of the words in the grammar has occured
    def OnRecognition(self, StreamNumber, StreamPosition, RecognitionType, Result):
        win = win32gui
        # Window title allows us to issue different keystrokes for different apps
        actwindow = win.GetWindowText (win.GetForegroundWindow())
        # newResult will be used to get recognised word text
        newResult = win32com.client.Dispatch(Result)
        word = newResult.PhraseInfo.GetText().lower()

###############################################
#       phrase recognition (by zahmeKatzen)
###############################################
        #Note: word can actually be a whole phrase
        #print word.encode("utf-8")

        if word == PROFILE.get_option("quit_phrase").lower():
            global RUN
            RUN = 0
            return

        if not COMMANDS.has_key(word):
            print "no such key: ", word
            return

        keystrokes = COMMANDS[word]

        if type(keystrokes) is not list:
            #make sure that keystrokes is a list!
            keystrokes = [keystrokes]

        for keypress in keystrokes:
            #simultaneous keypresses are seperated by "+"
            keys = keypress["keycodes"].split("+")
            repeat = 1
            if "repeat" in keypress.keys() and keypress["repeat"]!="0":
                repeat = int(keypress["repeat"]) #repeat=1 -> once
            for i in xrange(repeat):
                #press keys
                for kk in keys:
                    k = int(kk.strip())
                    win32api.keybd_event(k,0,0,0)
                    #wait a few miliseconds before pressing the next key
                    time.sleep(self.keypress_duration)

                #release keys
                for kk in keys:
                    k = int(kk.strip())
                    win32api.keybd_event(k,0,win32con.KEYEVENTF_KEYUP,0)


        print
        print "Recognized command:\t",
        print word.encode("utf-8") #yes, you get funny characters that way. I don't care anymore...

def main(wordsToAdd, commands):
    quit_phrase = PROFILE.get_option("quit_phrase")
    if quit_phrase: wordsToAdd.append(quit_phrase.lower())
    speechReco = SpeechRecognition(wordsToAdd)#

    dur = 0.0
    try:
        dur = float(PROFILE.get_option("keypress_duration"))
    except:
        print "Invalid value for keypress_duration",
        print PROFILE.get_option("keypress_duration")
    speechReco.eventHandler.keypress_duration = dur

    global COMMANDS
    global RUN
    RUN = 1
    COMMANDS = commands
    while RUN:
        pythoncom.PumpWaitingMessages()
        time.sleep(0.001)

if __name__=='__main__':
    #test-mode

    #feel free to change wordsToAdd to test other words and phrases
    wordsToAdd = [ "test", "best pest", "rest", "blessed", "blessed rest"]
    speechReco = SpeechRecognition(wordsToAdd)
    while 1:
        pythoncom.PumpWaitingMessages()





